/** emacs: this is -*- c++ -*- **/
/**
 **   @file    serialise_base.h        
 **                   
 **   @author  sutt
 **   @date    Tue 29 Dec 2020 13:57:14 GMT
 **
 **   $Id: serialise_base.h, v0.0   Tue 29 Dec 2020 13:57:14 GMT sutt $
 **
 **   Copyright (C) 2020 sutt (sutt@cern.ch)    
 **
 **/

#ifndef  SERIALISE_BASE_H
#define  SERIALISE_BASE_H

#include <vector>
#include <string>
#include <iostream>

#include <zlib.h>


namespace SB { 

  /// serialisation

  const double WRITEGUARD = 1234567890123456; 

  typedef uint64_t TYPE;  

  inline void serialise( std::vector<TYPE>& strm, const double& d ) { 
    strm.push_back( *((SB::TYPE*)&d) );
  } 

#if 0
  /// for some reason it cannot resolve the recursive template 
  /// for string and double
  template<typename T>
  inline void serialise( std::vector<std::vector<TYPE> >& strm, const std::vector<T>& v ) { 
    strm.push_back( v.size() );
    for ( size_t i=0 ; i<v.size() ; i++ ) serialise( strm, v[i] );
  } 
#endif

  
  inline void serialise( std::vector<TYPE>& strm, const std::string& s ) { 
    strm.push_back( s.size() );
    for ( size_t i=0 ; i<s.size() ; i++ ) strm.push_back( s[i] ); 
  } 


  inline void serialise( std::vector<TYPE>& strm, const std::vector<double>& v ) { 
    strm.push_back( v.size() );
    //    strm.insert( strm.end(), v.begin(), v.end() ); 
    for ( size_t i=0 ; i<v.size() ; i++ ) strm.push_back( *((SB::TYPE*)&v[i]) );
  } 
  
  inline void serialise( std::vector<TYPE>& strm, const std::vector<std::string>& v ) { 
    strm.push_back( v.size() );
    for ( size_t i=0 ; i<v.size() ; i++ ) serialise( strm, v[i] ); 
  }
 

  /// but it can for the vector<vector<T> > for some reason
  template<typename T>
  inline void serialise( std::vector<TYPE>& strm, const std::vector<std::vector<T> >& v ) { 
    strm.push_back( v.size() );
    for ( size_t i=0 ; i<v.size() ; i++ ) serialise( strm, v[i] );
  } 


  /// deserialise 

  inline void deserialise( std::vector<TYPE>::const_iterator& itr, double& v ) { 
    v = *((double*)&(*itr++));
  }

  inline void deserialise( std::vector<TYPE>::const_iterator& itr, std::vector<double>& v ) { 
    size_t n = (*itr++);
    v.clear();
    v.resize(n);
    for ( size_t i=0 ; i<n ; i++ ) v[i] = *((double*)&(*itr++)); //(*itr++);
  }

  inline void deserialise( std::vector<TYPE>::const_iterator& itr, std::vector<std::vector<double> >& v ) {
    size_t n = (*itr++);
    v.clear();
    v.resize(n);
    for ( size_t i=0 ; i<n ; i++ ) deserialise( itr, v[i] );
  } 

  inline void deserialise( std::vector<TYPE>::const_iterator& itr, std::string& s ) { 
    s.clear();
    size_t n = (*itr++);
    for ( size_t i=0 ; i<n ; i++ ) s += char(*itr++);
  }

  inline void deserialise( std::vector<TYPE>::const_iterator& itr, std::vector<std::string>& v ) { 
    size_t n = (*itr++);
    v.clear();
    v.resize(n);
    for ( size_t i=0 ; i<n ; i++ ) deserialise( itr, v[i] );
  }

  inline void deserialise( std::vector<TYPE>::const_iterator& itr, std::vector<std::vector<std::string> >& v ) {
    size_t n = (*itr++);
    v.clear();
    v.resize(n);
    for ( size_t i=0 ; i<n ; i++ ) deserialise( itr, v[i] );
  } 


}



#endif  // SERIALISE_BASE_H 



